import numpy as np
import astroquery
from astropy.io import fits
from astropy.wcs import WCS
from astroquery.skyview import SkyView
import matplotlib.pyplot as plt
import urllib

# Function to Get a Survey Image Provided the Object Position & Survey Name
# May need a downgrade of HTML5Lib: pip install --upgrade html5lib==1.0b8
def getSVImg(SC_ObjPos, SkyViewSurvey):
    """
    Input: a SkyCoord Position Vector and a Survey Type that is Compatable with NASA SkyView
    Output: A Numpy Pixel Array and Header Array
    """
    img_survey, pix_survey, hdr_survey = None, None, None
    try:
        img_survey = SkyView.get_images(position=SC_ObjPos, survey=[SkyViewSurvey],
                                        coordinates="J2000", pixels=30)
        if len(img_survey) > 0:
            pix_survey = img_survey[0][0].data
            hdr_survey = img_survey[0][0].header
    except (astroquery.exceptions.TimeoutError, urllib.error.HTTPError):
        pix_survey, hdr_survey = None, None
    return pix_survey, hdr_survey

if __name__ == "__main__":
    # Demo to Showcase that the SkyView image is working.
    Survey = 'DSS'
    ObjPos = 'Dumbbell Nebula'
    pixels, header = getSVImg(ObjPos, Survey)
    plt.imshow(pixels)
    plt.show()



""" Plot a K2 TPF and overlay a sky survey image."""

from __future__ import division, print_function, absolute_import

import argparse

from .tpf import TargetPixelFile
from .figure import K2Fig

def k2pix():
    """ Script to plot a K2 TPF and overlay a sky survey image."""
    parser = argparse.ArgumentParser(
        description="Plots a co-added Target Pixel File (TPF) from NASA's "
                    "Kepler/K2/TESS with a survey image overlaid.")
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='')
    parser.add_argument('--output', metavar='FILENAME',
                        type=str, default=None,
                        help='.gif or .mp4 output filename (default: gif with the same name as the input file)')
    parser.add_argument('--survey', metavar='SURVEY',
                        type=str, default=None,
                        help='survey (available in skyview) to overlay on the K2 image.')
    parser.add_argument('--stretch', type=str, default='log',
                        help='type of contrast stretching: "linear", "sqrt", '
                             '"power", "log", or "asinh" (default is "log")')
    parser.add_argument('--min_percent', metavar='%', type=float, default=1.,
                        help='minimum cut percentile (default: 1.0)')
    parser.add_argument('--max_percent', metavar='%', type=float, default=95.,
                        help='maximum cut percentile (default: 95)')
    parser.add_argument('--cmap', type=str,
                        default='gray', help='matplotlib color map name '
                                             '(default: gray)')
    parser.add_argument('--contour_color', type=str,
                        default='red', help='matplotlib color name '
                                             '(default: red)')
    parser.add_argument('tpf_filename', nargs='+',
                        help='path to one or more Target Pixel Files (TPF)')

    args = parser.parse_args()

    # # What is the data column to show?
    # if args.raw:
    #     data_col = 'RAW_CNTS'
    # elif args.background:
    #     data_col = 'FLUX_BKG'
    # elif args.cosmic:
    #     data_col = 'COSMIC_RAYS'
    # else:
    #     data_col = 'FLUX'

    for fn in args.tpf_filename:
        try:
            out_name = args.output
            if args.output is None:
                out_name = fn + '_overplot.png'
            tpf = TargetPixelFile(fn, verbose=args.verbose)
            fig = K2Fig(tpf)
            fig.create_figure(output_filename=out_name,
                              survey=args.survey,
                              stretch=args.stretch,
                              min_percent=args.min_percent,
                              max_percent=args.max_percent,
                              cmap=args.cmap,
                              contour_color=args.contour_color)
        except Exception as e:
            if args.verbose:
                raise e
            else:
                print('ERROR: {}'.format(e))

                 #
                #  output_filename, stretch='log', vmin=1, vmax=None,
                #                   cmap='gray', data_col='FLUX', min_percent=1,
                #                   max_percent=95

# Example use
if __name__ == '__main__':

    # fn = ('http://archive.stsci.edu/missions/kepler/target_pixel_files/'
    #       '0007/000757076/kplr000757076-2010174085026_lpd-targ.fits.gz')
    fn = ("https://archive.stsci.edu/missions/k2/target_pixel_files/c5/"
          "211900000/72000/ktwo211972478-c05_lpd-targ.fits.gz")
    tpf = TargetPixelFile(fn)
    fig = K2Fig(tpf)
    # Then run with all default arguments to generate image
    fig.create_figure("overplot.png","2MASS-K")



from __future__ import division, print_function, absolute_import

import imageio
import numpy as np
from tqdm import tqdm
import warnings

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.image import imsave
import matplotlib.patheffects as path_effects
from matplotlib.colors import NoNorm

from astropy import log
from astropy import visualization
from astropy.wcs import WCS

from .surveyquery import getSVImg

# Figure class
class K2Fig(object):
    """Figure showing K2 target pixel stamp and sky survey image."""

    def __init__(self,TPF):
        self.TPF = TPF
        self.verbose = self.TPF.verbose

    def cut_levels(self, min_percent=1., max_percent=95., data_col='FLUX'):
            """Determine the cut levels for contrast stretching.

            Returns
            -------
            vmin, vmax : float, float
                Min and max cut levels.
            """

            # Get co-added flux
            # update to use TPF
            sample = self.TPF.flux_binned()

            # Scale image
            with warnings.catch_warnings():
                warnings.filterwarnings('ignore', message="(.*)invalid value(.*)")
                vmin, vmax = np.percentile(sample[sample > 0],
                                           [min_percent, max_percent])
            return vmin, vmax



    # Set up the figure and axes using astropy WCS
    def create_figure(self, output_filename, survey, stretch='log', vmin=1, vmax=None, min_percent=1, max_percent=95,
                      cmap='gray', contour_color='red', data_col='FLUX'):
        """Returns a matplotlib Figure object that visualizes a frame.

        Parameters
        ----------

        vmin : float, optional
            Minimum cut level (default: 0).

        vmax : float, optional
            Maximum cut level (default: 5000).

        cmap : str, optional
            The matplotlib color map name.  The default is 'gray',
            can also be e.g. 'gist_heat'.

        raw : boolean, optional
            If `True`, show the raw pixel counts rather than
            the calibrated flux. Default: `False`.

        Returns
        -------
        image : array
            An array of unisgned integers of shape (x, y, 3),
            representing an RBG colour image x px wide and y px high.
        """
        # Get the flux data to visualize
        # Update to use TPF
        flx = self.TPF.flux_binned()
        # print(np.shape(flx))

        # calculate cut_levels
        if vmax is None:
            vmin, vmax = self.cut_levels(min_percent,max_percent,data_col)

        # Determine the figsize
        shape = list(flx.shape)
        # print(shape)
        # Create the figure and display the flux image using matshow
        fig = plt.figure(figsize=shape)
        # Display the image using matshow

        # Update to generate axes using WCS axes instead of plain axes
        ax = plt.subplot(projection=self.TPF.wcs)
        ax.set_xlabel('RA')
        ax.set_ylabel('Dec')

        if self.verbose:
            print('{} vmin/vmax = {}/{} (median={})'.format(data_col, vmin, vmax, np.nanmedian(flx)))

        if stretch == 'linear':
            stretch_fn = visualization.LinearStretch()
        elif stretch == 'sqrt':
            stretch_fn = visualization.SqrtStretch()
        elif stretch == 'power':
            stretch_fn = visualization.PowerStretch(1.0)
        elif stretch == 'log':
            stretch_fn = visualization.LogStretch()
        elif stretch == 'asinh':
            stretch_fn = visualization.AsinhStretch(0.1)
        else:
            raise ValueError('Unknown stretch: {0}.'.format(stretch))

        transform = (stretch_fn +
                     visualization.ManualInterval(vmin=vmin, vmax=vmax))
        ax.imshow((255*transform(flx)).astype(int), aspect='auto',
                   origin='lower', interpolation='nearest',
                   cmap=cmap, norm=NoNorm())
        ax.set_xticks([])
        ax.set_yticks([])

        current_ylims = ax.get_ylim()
        current_xlims = ax.get_xlim()

        pixels, header = getSVImg(self.TPF.position, survey)
        levels = np.linspace(np.min(pixels),np.percentile(pixels,95),10)
        ax.contour(pixels,transform=ax.get_transform(WCS(header)),
                    levels=levels,colors=contour_color)

        ax.set_xlim(current_xlims)
        ax.set_ylim(current_ylims)

        fig.canvas.draw()
        plt.savefig(output_filename, bbox_inches='tight', dpi=300)
        return fig



#!/usr/bin/env python
import sys
from setuptools import setup
# To use a consistent encoding
from codecs import open
import os
from os import path

# Prepare and send a new release to PyPI
if "release" in sys.argv[-1]:
    os.system("python setup.py sdist")
    os.system("twine upload dist/*")
    os.system("rm -rf dist/k2pix*")
    sys.exit()

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README'), encoding='utf-8') as f:
    long_description = f.read()

entry_points = {'console_scripts': ['k2pix = k2pix.main:k2pix']}

setup(
    name='k2pix',
    packages=['k2pix'],
    version='0.1',
    description="Overlaying a Sky View survey image's contours onto a K2 pixel stamp",
    long_description=long_description,
    url='https://github.com/gully/k2-pix',
    author='Stephanie Douglas and GitHub contributors',
    author_email='',
    license='MIT',
    entry_points=entry_points,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-cov'],
    keywords='astronomy astrophysics',
    install_requires=['pyketools', 'astroquery', 'numpy', 'matplotlib', 'tqdm', 'astropy']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""""Create movies or animated gifs from Kepler Target Pixel Files.

Author: Geert Barentsen, Stephanie Douglas, Erin Maier
"""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

__all__ = ["TargetPixelFile"]

import argparse
import imageio
import numpy as np
from tqdm import tqdm
import warnings

from astropy.io import fits
from astropy import log
from astropy import visualization
from astropy.wcs import WCS
import astropy.units as u
from astropy import coordinates as coords

###
# Kepler constants
###

# Quality flags, from the Kepler Archive Manual
KEPLER_QUALITY_FLAGS = {
    "1": "Attitude tweak",
    "2": "Safe mode",
    "4": "Coarse point",
    "8": "Earth point",
    "16": "Zero crossing",
    "32": "Desaturation event",
    "64": "Argabrightening",
    "128": "Cosmic ray",
    "256": "Manual exclude",
    "1024": "Sudden sensitivity dropout",
    "2048": "Impulsive outlier",
    "4096": "Argabrightening",
    "8192": "Cosmic ray",
    "16384": "Detector anomaly",
    "32768": "No fine point",
    "65536": "No data",
    "131072": "Rolling band",
    "262144": "Rolling band",
    "524288": "Possible thruster firing",
    "1048576": "Thruster firing"
}

###
# Fix TPF header WCS keywords
###

def k2_ConvertHeaderWCS(tpf_header):
    """
    Reads the WCS for the raw pixel counts into astropy.wcs
    and returns the WCS.
    """
    funny_keywords = {'1CTYP4': 'CTYPE1',
                      '2CTYP4': 'CTYPE2',
                      '1CRPX4': 'CRPIX1',
                      '2CRPX4': 'CRPIX2',
                      '1CRVL4': 'CRVAL1',
                      '2CRVL4': 'CRVAL2',
                      '1CUNI4': 'CUNIT1',
                      '2CUNI4': 'CUNIT2',
                      '1CDLT4': 'CDELT1',
                      '2CDLT4': 'CDELT2',
                      '11PC4': 'PC1_1',
                      '12PC4': 'PC1_2',
                      '21PC4': 'PC2_1',
                      '22PC4': 'PC2_2'}
    mywcs = {}
    for oldkey, newkey in funny_keywords.items():
        mywcs[newkey] = tpf_header[oldkey]

    return WCS(mywcs)

###
# Helper classes
###

class BadKeplerFrame(Exception):
    """Raised if a frame is empty."""
    pass


class BadCadenceRange(Exception):
    """Raised if a the range of frames is invalid."""
    pass


class TargetPixelFile(object):
    """Represent a Target Pixel File (TPC) from the Kepler spacecraft.

    Parameters
    ----------
    filename : str
        Path of the pixel file.

    cache : boolean
        If the file name is a URL, this specifies whether or not to save the
        file locally in Astropy’s download cache. (default: True)

    verbose : boolean
        If True, print debugging output.
    """
    def __init__(self, filename, cache=True, verbose=False):
        self.filename = filename
        self.hdulist = fits.open(filename, cache=cache)
        self.header = self.hdulist[1].header
        self.no_frames = len(self.hdulist[1].data['FLUX'])
        self.verbose = verbose
        if not self.verbose:
            import warnings
            warnings.filterwarnings("ignore")

    @property
    def objectname(self):
        try:
            return self.hdulist[0].header['OBJECT']
        except KeyError:
            return ''

    @property
    def ra(self):
        return self.hdulist[0].header['RA_OBJ']

    @property
    def dec(self):
        return self.hdulist[0].header['DEC_OBJ']

    @property
    def wcs(self):
        return k2_ConvertHeaderWCS(self.header)

    @property
    def position(self):
        return coords.SkyCoord(self.ra,self.dec,unit=u.deg,frame="icrs")

    @property
    def flux(self):
        flx = np.array(self.hdulist[1].data["FLUX"])

        bad_frames = np.zeros(len(flx),bool)

        # delete bad frames
        for frameno in np.arange(self.no_frames):
            flags = self.quality_flags(frameno)
            if len(flags)>0:
                bad_frames[frameno] = True

            infinites = np.isfinite(flx[frameno])
            fin_med = np.nanmedian(flx[frameno][infinites==False])
            flx[frameno][infinites==False] = fin_med

        good_flux = np.delete(flx, np.where(bad_frames)[0], axis=0)
        return good_flux

    def flux_binned(self):
        flux_series = self.flux
        return np.sum(flux_series,axis=0)

    def quality_flags(self, frameno):
        """Returns a list of strings describing the quality flags raised."""
        quality = self.hdulist[1].data['QUALITY'][frameno]
        flags = []
        for flag in KEPLER_QUALITY_FLAGS.keys():
            if quality & int(flag) > 0:
                flags.append(KEPLER_QUALITY_FLAGS[flag])
        return flags
